@@ -4,12 +4,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <locale>
+#include <sstream>
+#include <string>
 #include <Eigen/Core>
 
+#include "fp16.h"
 #include "init.h"
 #include "util.h"
 
-#if defined(__DARWIN__)
+#if defined(__APPLE__)
 #include <OpenGL/OpenGL.h>
 #elif defined(WIN32)
 #include <epoxy/wgl.h>
@@ -83,17 +87,41 @@ string read_file(const string &filename)
 {
 	const string full_pathname = *movit_data_directory + "/" + filename;
 
-	static char buf[131072];
 	FILE *fp = fopen(full_pathname.c_str(), "r");
 	if (fp == NULL) {
 		perror(full_pathname.c_str());
 		exit(1);
 	}
 
-	int len = fread(buf, 1, sizeof(buf), fp);
+	int ret = fseek(fp, 0, SEEK_END);
+	if (ret == -1) {
+		perror("fseek(SEEK_END)");
+		exit(1);
+	}
+
+	int size = ftell(fp);
+
+	ret = fseek(fp, 0, SEEK_SET);
+	if (ret == -1) {
+		perror("fseek(SEEK_SET)");
+		exit(1);
+	}
+
+	string str;
+	str.resize(size);
+	ret = fread(&str[0], size, 1, fp);
+	if (ret == -1) {
+		perror("fread");
+		exit(1);
+	}
+	if (ret == 0) {
+		fprintf(stderr, "Short read when trying to read %d bytes from %s\n",
+		        size, full_pathname.c_str());
+		exit(1);
+	}
 	fclose(fp);
 
-	return string(buf, len);
+	return str;
 }
 
 string read_version_dependent_file(const string &base, const string &extension)
@@ -144,34 +172,60 @@ void print_3x3_matrix(const Eigen::Matrix3d& m)
 
 string output_glsl_mat3(const string &name, const Eigen::Matrix3d &m)
 {
-	char buf[1024];
-	sprintf(buf,
-		"const mat3 %s = mat3(\n"
-		"    %.8f, %.8f, %.8f,\n"
-		"    %.8f, %.8f, %.8f,\n"
-		"    %.8f, %.8f, %.8f);\n\n",
-		name.c_str(),
-		m(0,0), m(1,0), m(2,0),
-		m(0,1), m(1,1), m(2,1),
-		m(0,2), m(1,2), m(2,2));
-	return buf;
+	// Use stringstream to be independent of the current locale in a thread-safe manner.
+	stringstream ss;
+	ss.imbue(locale("C"));
+	ss.precision(8);
+	ss << "const mat3 " << name << " = mat3(\n";
+	ss << "    " << m(0,0) << ", " << m(1,0) << ", " << m(2,0) << ",\n";
+	ss << "    " << m(0,1) << ", " << m(1,1) << ", " << m(2,1) << ",\n";
+	ss << "    " << m(0,2) << ", " << m(1,2) << ", " << m(2,2) << ");\n\n";
+	return ss.str();
 }
 
-void combine_two_samples(float w1, float w2, float *offset, float *total_weight, float *sum_sq_error)
+string output_glsl_vec2(const string &name, float x, float y)
+{
+	// Use stringstream to be independent of the current locale in a thread-safe manner.
+	stringstream ss;
+	ss.imbue(locale("C"));
+	ss.precision(8);
+	ss << "const vec2 " << name << " = vec2(" << x << ", " << y << ");\n";
+	return ss.str();
+}
+
+string output_glsl_vec3(const string &name, float x, float y, float z)
+{
+	// Use stringstream to be independent of the current locale in a thread-safe manner.
+	stringstream ss;
+	ss.imbue(locale("C"));
+	ss.precision(8);
+	ss << "const vec3 " << name << " = vec3(" << x << ", " << y << ", " << z << ");\n";
+	return ss.str();
+}
+
+template<class DestFloat>
+void combine_two_samples(float w1, float w2, float pos1, float pos2, unsigned size,
+                         DestFloat *offset, DestFloat *total_weight, float *sum_sq_error)
 {
 	assert(movit_initialized);
 	assert(w1 * w2 >= 0.0f);  // Should not have differing signs.
-	float z;  // Just a shorter name for offset.
+	float z;  // Normalized 0..1 between pos1 and pos2.
 	if (fabs(w1 + w2) < 1e-6) {
 		z = 0.5f;
 	} else {
 		z = w2 / (w1 + w2);
 	}
 
+	// Round to the desired precision. Note that this might take z outside the 0..1 range.
+	*offset = from_fp64<DestFloat>(pos1 + z * (pos2 - pos1));
+	z = (to_fp64(*offset) - pos1) / (pos2 - pos1);
+
 	// Round to the minimum number of bits we have measured earlier.
 	// The card will do this for us anyway, but if we know what the real z
 	// is, we can pick a better total_weight below.
-	z = lrintf(z / movit_texel_subpixel_precision) * movit_texel_subpixel_precision;
+	z *= size;  // Move to pixel coordinates,
+	z = lrintf(z / movit_texel_subpixel_precision) * movit_texel_subpixel_precision;  // Round.
+	z /= size;  // Move back to normalized coordinates.
 	
 	// Choose total weight w so that we minimize total squared error
 	// for the effective weights:
@@ -187,18 +241,23 @@ void combine_two_samples(float w1, float w2, float *offset, float *total_weight,
 	//
 	// If z had infinite precision, this would simply reduce to w = w1 + w2.
 	*total_weight = (w1 * (1 - z) + w2 * z) / (z * z + (1 - z) * (1 - z));
-	*offset = z;
 
 	if (sum_sq_error != NULL) {
 		float err1 = *total_weight * (1 - z) - w1;
 		float err2 = *total_weight * z - w2;
 		*sum_sq_error = err1 * err1 + err2 * err2;
 	}
-
-	assert(*offset >= 0.0f);
-	assert(*offset <= 1.0f);
 }
 
+// Explicit instantiations.
+template
+void combine_two_samples<float>(float w1, float w2, float pos1, float pos2, unsigned size,
+                                float *offset, float *total_weight, float *sum_sq_error);
+
+template
+void combine_two_samples<fp16_int_t>(float w1, float w2, float pos1, float pos2, unsigned size,
+                                     fp16_int_t *offset, fp16_int_t *total_weight, float *sum_sq_error);
+
 GLuint fill_vertex_attribute(GLuint glsl_program_num, const string &attribute_name, GLint size, GLenum type, GLsizeiptr data_size, const GLvoid *data)
 {
 	int attrib = glGetAttribLocation(glsl_program_num, attribute_name.c_str());
@@ -256,7 +315,7 @@ unsigned next_power_of_two(unsigned v)
 
 void *get_gl_context_identifier()
 {
-#if defined(__DARWIN__)
+#if defined(__APPLE__)
 	return (void *)CGLGetCurrentContext();
 #elif defined(WIN32)
 	return (void *)wglGetCurrentContext();
